# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from rapido.plone.testing import RAPIDO_PLONE_INTEGRATION_TESTING  # noqa
from plone import api

import unittest2 as unittest


class TestSetup(unittest.TestCase):
    """Test that rapido.plone is properly installed."""

    layer = RAPIDO_PLONE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer['portal']
        self.installer = api.portal.get_tool('portal_quickinstaller')

    def test_product_installed(self):
        """Test if rapido.plone is installed with portal_quickinstaller."""
        self.assertTrue(self.installer.isProductInstalled('rapido.plone'))

    def test_uninstall(self):
        """Test if rapido.plone is cleanly uninstalled."""
        self.installer.uninstallProducts(['rapido.plone'])
        self.assertFalse(self.installer.isProductInstalled('rapido.plone'))



from AccessControl import Unauthorized as unauth
from datetime import datetime
import json
from plone.protect import CheckAuthenticator
from plone.protect.authenticator import createToken
from Products.Five.browser import BrowserView
import zExceptions
from zope.interface import implements
from zope.publisher.interfaces import IPublishTraverse

from rapido.core.exceptions import NotAllowed, Unauthorized, NotFound
from rapido.core.interfaces import IRest, IDisplay
from rapido.plone.app import get_app


class PythonObjectEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime):
            return obj.isoformat()
        if isinstance(
            obj,
            (list, dict, str, unicode, int, float, bool, type(None))
        ):
            return json.JSONEncoder.default(self, obj)
        return {'_not_serializable': str(obj)}


class RapidoView(BrowserView):
    implements(IPublishTraverse)

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.method = self.request.method
        self.path = []

    def publishTraverse(self, request, name):
        self.path.append(name)
        return self

    def store_app_messages(self, app):
        if not app.settings.get('debug'):
            return
        if len(app.messages) > 0:
            old_messages = app.context.cache.get('messages', [])
            app.context.cache['messages'] = old_messages + app.messages

    def get_app_messages(self):
        app_id = self.path[0]
        app = get_app(app_id, self.request)
        messages = app.context.cache.get('messages', [])
        app.context.cache['messages'] = []
        return messages

    def content(self, path=None):
        if not path:
            path = self.path
        app_id = path[0]
        app = get_app(app_id, self.request)
        method = getattr(IDisplay(app), self.method)
        try:
            (result, redirect) = method(path, self.request)
        except NotAllowed:
            raise zExceptions.BadRequest()
        except NotFound:
            raise zExceptions.NotFound()
        except Unauthorized:
            raise unauth("Not authorized")
        self.store_app_messages(app)
        if redirect:
            self.request.RESPONSE.redirect(redirect)
        else:
            return result

    def json(self, path=None):
        if not path:
            path = self.path
        app_id = path[0]
        if len(path) > 1:
            path = path[1:]
        else:
            path = None
        try:
            app = get_app(app_id, self.request)
            if self.method not in ["GET", "OPTIONS"]:
                CheckAuthenticator(self.request)
            method = getattr(IRest(app), self.method)
            result = method(path, self.request.get('BODY'))
            self.store_app_messages(app)
            return result
        except NotAllowed:
            self.request.response.setStatus(403)
        except NotFound, e:
            self.request.response.setStatus(404)
            return {'id': str(e.name)}
        except Unauthorized:
            self.request.response.setStatus(401)
        except Exception, e:
            self.request.response.setStatus(500)
            return {'error': str(e)}

    def __call__(self):
        if self.path[0] == 'view':
            # this is neutral, we just return the default content view
            # but it gives the opportunity to create specific pseudo views
            # via our Diazo rules.xml
            return self.context()

        self.request.response.setHeader('X-Theme-Disabled', '1')

        if self.path[-1].startswith('$'):
            # inject Diazo parent request path
            parent_path = self.request.PARENT_REQUEST['URL'].split('/')
            try:
                offset = int(self.path[-1][1:])
            except ValueError:
                return "Bad Rapido url injection %s" % self.path[-1]
            index = parent_path.index('@@rapido')
            self.path = self.path[:-1] + parent_path[index + offset:]

        if len(self.path) == 2 and self.path[1] == '_log':
            messages = self.get_app_messages()
            self.request.response.setHeader('content-type', 'application/json')
            return json.dumps(messages, cls=PythonObjectEncoder)

        if "application/json" in self.request.getHeader('Accept', ''):
            result = self.json()
            if len(self.path) == 1:
                self.request.response.setHeader('X-CSRF-TOKEN', createToken())
            self.request.response.setHeader('content-type', 'application/json')
            return json.dumps(result, cls=PythonObjectEncoder)
        else:
            result = self.content()
            return result

    def __contains__(self, name):
        # When processing other methods than GET and POST, ZPublisher tries to
        # make sure the requested resource exists in the parent. So we just
        # pretend that any part of the url path is an existing element.
        # That's quite hacky :)
        if name in self.request.URL.split('/'):
            return True
        else:
            return False



def display_quote(context):
    doc = context.app.get_record('doc1')
    if doc:
        return doc.get('quote', '')
    else:
        return "No quote"


def create(context):
    doc = context.app.create_record('doc1')
    doc['block'] = 'action'
    doc["quote"] = "Knowledge is power, France is bacon."
    doc.save()


def boom(context):
    context.wrong()


def log_me(context):
    context.app.log("Hello!")
    context.app.log([1, 2, {'a': 3}])
    context.app.log(context)


def create_content(context):
    context.api.content.create(
        container=context.api.portal.get(),
        type='Document',
        title='My Content',
    )



from plone import api
from plone.app.theming.interfaces import THEME_RESOURCE_NAME
from plone.app.theming.utils import getCurrentTheme
from plone.memoize.interfaces import ICacheChooser
from plone.resource.utils import queryResourceDirectory
from zope.component import queryUtility
from zope.interface import implements
from zope.pagetemplate.pagetemplate import PageTemplate
from zExceptions import NotFound

from rapido.core import exceptions
from rapido.core.app import Context
from rapido.core.interfaces import IRapidable, IRapidoApplication


class RapidoTemplateFile(PageTemplate):

    def __init__(self, text):
        self.write(text)

    def __call__(self, elements, context):
        try:
            return self.pt_render({'elements': elements, 'context': context})
        except Exception, e:
            error = str(e)
            extra = ('<pre>' in error and error[
                error.index('<pre>') + 5:error.index('</pre>')]) or ''
            lines = error.split('\n')
            if len(lines) > 4:
                message = "%s\n%s" % (lines[2], lines[4])
            else:
                message = str(e)
            return "<pre>Rendering error\n%s\n\n%s</pre>" % (
                message, extra.replace('\\n', '\n'))


class RapidoApplication(object):
    implements(IRapidable)

    def __init__(self, id, context):
        self.id = id
        self.context = context
        self.available_rules = {}
        self.resources = self.get_resource_directory()
        self.cache = queryUtility(ICacheChooser)("rapido-" + id)

    def url(self):
        url_format = "%s/@@rapido/%s"
        target = self.context.content or api.portal.get()
        return url_format % (
            target.absolute_url(),
            self.id,
        )

    @property
    def root(self):
        return api.portal.get()

    @property
    def blocks(self):
        ids = []
        files = self.resources['blocks'].listDirectory()
        for filename in files:
            id = filename.rpartition('.')[0]
            if id not in ids:
                ids.append(id)
        return ids

    def get_settings(self):
        try:
            return self.get_resource('settings.yaml')
        except KeyError:
            # settings.yaml is not mandatory
            return 'no_settings: {}'

    def get_block(self, block_id, ftype='yaml'):
        path = "blocks/%s.%s" % (block_id, ftype)
        try:
            content = self.get_resource(path)
            if ftype in ('html', 'pt', ):
                content = content.decode('utf-8')
            return content
        except KeyError:
            if ftype == "yaml":
                # the YAML file is not mandatory, return a default content
                return 'id: %s' % block_id
            elif ftype == "html":
                # if .html is missing, try .pt
                try:
                    pt = self.get_block(block_id, ftype="pt")
                    return RapidoTemplateFile(pt)
                except KeyError:
                    raise KeyError('%s.%s' % (block_id, ftype))
            else:
                raise KeyError('%s.%s' % (block_id, ftype))

    def get_resource_directory(self):
        theme = getCurrentTheme()
        directory = queryResourceDirectory(THEME_RESOURCE_NAME, theme)
        try:
            return directory['rapido'][self.id]
        except NotFound:
            raise exceptions.NotFound(self.id)

    def get_resource(self, path):
        try:
            return self.resources.readFile(str(path))
        except:
            raise KeyError(path)

    def current_user(self):
        """ Returns the current user id
        """
        return api.user.get_current().getUserName()

    def current_user_groups(self):
        """ Get the current user groups
        """
        if api.user.is_anonymous():
            return []
        member = api.user.get_current()
        return member.getGroups()

    def is_manager(self):
        if api.user.is_anonymous():
            return False
        if 'Manager' in api.user.get_roles():
            return True
        return False


def get_app(app_id, request, content=None):
    portal = api.portal.get()
    context = Context()
    context.request = request
    context.parent_request = request.get("PARENT_REQUEST", None)
    if context.parent_request:
        path = '/'.join(
            context.parent_request.physicalPathFromURL(
                context.parent_request.URL))
    else:
        path = '/'.join(request.physicalPathFromURL(request.URL))
    path = path.split("@@")[0]
    context.content = content
    while not hasattr(context.content, 'portal_type'):
        try:
            context.content = portal.unrestrictedTraverse(path)
        except:
            pass
        path = '/'.join(path.split('/')[0:-1])
    context.portal = portal
    context.api = api
    context.rapido = lambda id, content=context.content: get_app(
        id, request, content=content)
    app = RapidoApplication(app_id, context)
    return IRapidoApplication(app)



from OFS.SimpleItem import SimpleItem
from plone import api
from plone.app.contentrules.actions import ActionAddForm, ActionEditForm
from plone.app.contentrules.browser.formhelper import ContentRuleFormWrapper
from plone.contentrules.rule.interfaces import IExecutable, IRuleElementData
from zope.component import adapts
from zope.interface import implements, Interface
from zope import schema

from rapido.core.exceptions import NotFound
from rapido.plone import _
from rapido.plone.app import get_app


class IAction(Interface):
    """Interface for the configurable aspects of a rapido action.

    This is also used to create add and edit forms, below.
    """

    app = schema.TextLine(title=_(u"Rapido application"),
        description=_(u"The targeted Rapido application."),
        required=True)

    block = schema.TextLine(title=_(u"Block"),
        description=_(u"The block providing the method."),
        required=True)

    method = schema.TextLine(title=_(u"Method"),
        description=_(u"The name of the method to execute."),
        required=True)


class Action(SimpleItem):
    """The actual persistent implementation of the action element.
    """
    implements(IAction, IRuleElementData)

    app = ''
    block = ''
    method = ''

    element = 'rapido.plone.Action'

    @property
    def summary(self):
        return _(u"Call Rapido method %s from %s/%s" % (
            self.method, self.app, self.block))


class ActionExecutor(object):
    """The executor for this action.
    """
    implements(IExecutable)
    adapts(Interface, IAction, Interface)

    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event

    def __call__(self):
        request = self.context.REQUEST
        try:
            app = get_app(self.element.app, request, content=self.event.object)
        except NotFound:
            api.portal.show_message(
                message="Rapido application %s cannot be found." % (
                    self.element.app,),
                request=request,
                type='error',
            )
            return True
        block = app.get_block(self.element.block)
        block.compute_element(self.element.method, {'block': block})
        return True


class AddForm(ActionAddForm):
    """An add form for rapido action.
    """
    schema = IAction
    label = _(u"Add Rapido Action")
    description = _(u"A Rapido action executes a Rapido method.")
    form_name = _(u"Configure element")
    Type = Action


class AddFormView(ContentRuleFormWrapper):
    form = AddForm


class EditForm(ActionEditForm):
    """An edit form for rapido action.
    """
    schema = IAction
    label = _(u"Edit Rapido Action")
    description = _(u"A Rapido action executes a Rapido method.")
    form_name = _(u"Configure element")


class EditFormView(ContentRuleFormWrapper):
    form = EditForm



# -*- coding: utf-8 -*-
import Globals
from plone.app.theming.interfaces import IThemeSettings
from plone.app.theming.utils import applyTheme
from plone.app.theming.utils import getTheme
from plone.registry.interfaces import IRegistry
import unittest2 as unittest
from zope.component import getUtility
from zope.publisher.browser import TestRequest

from rapido.plone.app import get_app
from rapido.plone.testing import RAPIDO_PLONE_FUNCTIONAL_TESTING


class FakeRequest(TestRequest):

    def physicalPathFromURL(self, url):
        return []


class TestCase(unittest.TestCase):

    layer = RAPIDO_PLONE_FUNCTIONAL_TESTING

    def setUp(self):
        # Enable debug mode always to ensure cache is disabled by default
        Globals.DevelopmentMode = True

        self.settings = getUtility(IRegistry).forInterface(IThemeSettings)
        self.settings.enabled = True
        theme = getTheme('rapido.plone.tests')
        applyTheme(theme)

        import transaction
        transaction.commit()

        self.portal = self.layer['portal']

    def tearDown(self):
        Globals.DevelopmentMode = False

    def test_app_url(self):
        request = FakeRequest()
        app = get_app('testapp', request)
        self.assertEquals(app.url,
            'http://localhost:55001/plone/@@rapido/testapp')



# -*- coding: utf-8 -*-
from plone.app.theming.interfaces import IThemeSettings
from plone.app.theming.utils import applyTheme
from plone.app.theming.utils import getTheme
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.registry.interfaces import IRegistry
from plone.testing.z2 import Browser
from zope.component import getUtility
import Globals
import unittest2 as unittest

from rapido.plone.testing import RAPIDO_PLONE_FUNCTIONAL_TESTING


class TestCase(unittest.TestCase):

    layer = RAPIDO_PLONE_FUNCTIONAL_TESTING

    def setUp(self):
        # Enable debug mode always to ensure cache is disabled by default
        Globals.DevelopmentMode = True

        self.settings = getUtility(IRegistry).forInterface(IThemeSettings)
        self.settings.enabled = True
        theme = getTheme('rapido.plone.tests')
        applyTheme(theme)

        import transaction
        transaction.commit()

        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False

    def tearDown(self):
        Globals.DevelopmentMode = False

    def test_view(self):
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/view/whatever')
        self.assertTrue("This is the theme" in self.browser.contents)

    def test_path_from_parent_request(self):
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/view/anything/testapp/blocks/basic')
        self.assertTrue("How simple is it" in self.browser.contents)

    def test_refresh(self):
        self.browser.addHeader(
            'Authorization',
            'Basic %s:%s' % (SITE_OWNER_NAME, SITE_OWNER_PASSWORD,)
        )
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/testapp/refresh')
        self.assertTrue("Refreshed" in self.browser.contents)



# -*- coding: utf-8 -*-
from plone.app.theming.interfaces import IThemeSettings
from plone.app.theming.utils import applyTheme
from plone.app.theming.utils import getTheme
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID, TEST_USER_PASSWORD
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.registry.interfaces import IRegistry
from plone.testing.z2 import Browser
from zope.component import getUtility
import Globals
import unittest2 as unittest

from rapido.plone.testing import RAPIDO_PLONE_FUNCTIONAL_TESTING


class TestCase(unittest.TestCase):

    layer = RAPIDO_PLONE_FUNCTIONAL_TESTING

    def setUp(self):
        # Enable debug mode always to ensure cache is disabled by default
        Globals.DevelopmentMode = True

        self.settings = getUtility(IRegistry).forInterface(IThemeSettings)
        self.settings.enabled = True
        theme = getTheme('rapido.plone.tests')
        applyTheme(theme)

        import transaction
        transaction.commit()

        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False
        self.browser.raiseHttpErrors = False
        self.browser.addHeader('Accept', 'application/json')

    def tearDown(self):
        Globals.DevelopmentMode = False

    def test_refresh_no_token(self):
        self.browser.addHeader(
            'Authorization',
            'Basic %s:%s' % (SITE_OWNER_NAME, SITE_OWNER_PASSWORD,)
        )
        self.browser.post(
            self.portal.absolute_url() + '/@@rapido/testapp/refresh', '')
        self.assertEquals(self.browser.headers["status"],
            '500 Internal Server Error')
        self.assertEquals(self.browser.contents,
            '{"error": "Form authenticator is invalid."}')

    def test_refresh_not_manager(self):
        self.browser.addHeader(
            'Authorization',
            'Basic %s:%s' % (TEST_USER_ID, TEST_USER_PASSWORD,)
        )
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/testapp')
        self.assertTrue('x-csrf-token' in self.browser.headers)
        token = self.browser.headers['x-csrf-token']
        self.browser.addHeader('x-csrf-token', token)
        self.browser.post(
            self.portal.absolute_url() + '/@@rapido/testapp/refresh', '')
        self.assertEquals(self.browser.headers["status"],
            '401 Unauthorized')

    def test_refresh(self):
        self.browser.addHeader(
            'Authorization',
            'Basic %s:%s' % (SITE_OWNER_NAME, SITE_OWNER_PASSWORD,)
        )
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/testapp')
        self.assertTrue('x-csrf-token' in self.browser.headers)
        token = self.browser.headers['x-csrf-token']
        self.browser.addHeader('x-csrf-token', token)
        self.browser.post(
            self.portal.absolute_url() + '/@@rapido/testapp/refresh', '')
        self.assertEquals(self.browser.headers["status"],
            '200 Ok')
        self.assertEquals(self.browser.contents,
            '{"success": "refresh", "indexes": ["id"]}')

    def test_403(self):
        self.browser.addHeader(
            'Authorization',
            'Basic %s:%s' % (SITE_OWNER_NAME, SITE_OWNER_PASSWORD,)
        )
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/testapp')
        self.assertTrue('x-csrf-token' in self.browser.headers)
        token = self.browser.headers['x-csrf-token']
        self.browser.addHeader('x-csrf-token', token)
        self.browser.post(
            self.portal.absolute_url() + '/@@rapido/testapp/wrong', '')
        self.assertEquals(self.browser.headers["status"],
            '403 Forbidden')

    def test_404(self):
        self.browser.addHeader(
            'Authorization',
            'Basic %s:%s' % (SITE_OWNER_NAME, SITE_OWNER_PASSWORD,)
        )
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/testapp')
        self.assertTrue('x-csrf-token' in self.browser.headers)
        token = self.browser.headers['x-csrf-token']
        self.browser.addHeader('x-csrf-token', token)
        self.browser.post(
            self.portal.absolute_url() + '/@@rapido/testapp/record/unknown',
            '')
        self.assertEquals(self.browser.headers["status"],
            '404 Not Found')

    def test_401(self):
        self.browser.addHeader(
            'Authorization',
            'Basic %s:%s' % (TEST_USER_ID, TEST_USER_PASSWORD,)
        )
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/testapp')
        self.assertTrue('x-csrf-token' in self.browser.headers)
        token = self.browser.headers['x-csrf-token']
        self.browser.addHeader('x-csrf-token', token)
        self.browser.post(
            self.portal.absolute_url() + '/@@rapido/testapp/refresh',
            '')
        self.assertEquals(self.browser.headers["status"],
            '401 Unauthorized')



from Products.Five.browser import BrowserView

from rapido.plone.app import get_app


class RapidoCall(BrowserView):
    """Call a Rapido element.

    Useful in PythonScripts
    """

    def __call__(self, path, content=None, **kw):
        (app_id, block_id, element_id) = path.split('/')
        app = get_app(app_id, self.request, content=content)
        block = app.get_block(block_id)
        return block.compute_element(
            element_id, {'block': block, 'params': kw})



# -*- coding: utf-8 -*-
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2
from zope.configuration import xmlconfig


class RapidoPloneLayer(PloneSandboxLayer):

    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        import rapido.plone.tests
        xmlconfig.file(
            'configure.zcml',
            rapido.plone.tests,
            context=configurationContext
        )
        # Run the startup hook
        from plone.app.theming.plugins.hooks import onStartup
        onStartup(None)

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'rapido.plone:default')


RAPIDO_PLONE_FIXTURE = RapidoPloneLayer()


RAPIDO_PLONE_INTEGRATION_TESTING = IntegrationTesting(
    bases=(RAPIDO_PLONE_FIXTURE,),
    name='RapidoPloneLayer:IntegrationTesting'
)


RAPIDO_PLONE_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(RAPIDO_PLONE_FIXTURE, z2.ZSERVER_FIXTURE),
    name='RapidoPloneLayer:FunctionalTesting'
)


RAPIDO_PLONE_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(
        RAPIDO_PLONE_FIXTURE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        z2.ZSERVER_FIXTURE
    ),
    name='RapidoPloneLayer:AcceptanceTesting'
)



from plone.memoize.view import memoize
from plone.supermodel import model
from plone.tiles import Tile
from zope import schema

from rapido.plone import _


class IRapidoTile(model.Schema):

    path = schema.TextLine(
        title=_(u"Rapido path"),
        description=_(u"Enter a valid rapido url "
            "(without the @@rapido/ prefix)."),
        required=True,
    )


class RapidoTile(Tile):
    """Rapido tile
    """

    @property
    @memoize
    def content(self):
        rapido = self.context.unrestrictedTraverse("@@rapido")
        return rapido.content(self.data['path'].split('/'))



# -*- coding: utf-8 -*-
from AccessControl import Unauthorized
import json
from plone.app.testing import (
    TEST_USER_ID,
    TEST_USER_PASSWORD,
    SITE_OWNER_NAME,
    SITE_OWNER_PASSWORD,
)
from plone.app.theming.interfaces import IThemeSettings
from plone.app.theming.utils import applyTheme
from plone.app.theming.utils import getTheme
from plone.registry.interfaces import IRegistry
from plone.testing.z2 import Browser
import zExceptions
from zope.component import getUtility
import Globals
import unittest2 as unittest

from rapido.core.exceptions import ExecutionError
from rapido.plone.testing import RAPIDO_PLONE_FUNCTIONAL_TESTING


class TestCase(unittest.TestCase):

    layer = RAPIDO_PLONE_FUNCTIONAL_TESTING

    def setUp(self):
        # Enable debug mode always to ensure cache is disabled by default
        Globals.DevelopmentMode = True

        self.settings = getUtility(IRegistry).forInterface(IThemeSettings)
        self.settings.enabled = True
        theme = getTheme('rapido.plone.tests')
        applyTheme(theme)

        import transaction
        transaction.commit()

        self.portal = self.layer['portal']
        self.browser = Browser(self.layer['app'])
        self.browser.handleErrors = False
        self.browser.raiseHttpErrors = False

    def tearDown(self):
        Globals.DevelopmentMode = False

    def test_static_block(self):
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/otherapp/blocks/purehtml')
        self.assertTrue("""<p>Hello, I am static</p>"""
            in self.browser.contents)

    def test_traverse_to_content(self):
        self.browser.open(
            self.portal.absolute_url() +
            '/sendto_form/@@rapido/otherapp/blocks/purehtml')
        self.assertTrue("""<p>Hello, I am static</p>"""
            in self.browser.contents)

    def test_block_with_basic_element(self):
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/testapp/blocks/basic')
        self.assertTrue("""<span>How simple is it? Very simple!</span>"""
            in self.browser.contents)

    def test_block_with_text_element(self):
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/testapp/blocks/text')
        self.assertTrue('<span>How simple is it? <input type="text"\n'
            '        name="answer" value="" /></span>'
            in self.browser.contents)

    def test_block_with_action_element(self):
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/testapp/blocks/action')
        self.assertTrue("""<span>Quote is: No quote</span>"""
            in self.browser.contents)
        self.assertTrue('<input type="submit"\n        name="action.create"'
            ' value="Create a quote" />'
            in self.browser.contents)
        self.browser.getControl('Create a quote').click()
        self.assertTrue('<span>Quote is: Knowledge is power, France is bacon.'
            '</span>'
            in self.browser.contents)

    def test_error(self):
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/testapp/_log')
        self.assertEquals(self.browser.contents, '[]')
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/testapp/blocks/action')
        self.assertRaises(
            ExecutionError,
            self.browser.getControl('Make an error').click
        )

    def test_log(self):
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/testapp/_log')
        self.assertEquals(self.browser.contents, '[]')
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/testapp/blocks/action')
        self.browser.getControl('Write a log').click()
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/testapp/_log')
        messages = json.loads(self.browser.contents)
        self.assertEquals(messages[0], "Hello!")
        self.assertEquals(messages[1], [1, 2, {"a": 3}])
        self.assertTrue("_not_serializable" in messages[2])

    def test_save_anonymous(self):
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/testapp/blocks/book')
        self.assertRaises(
            Unauthorized,
            self.browser.getControl('Save').click,
        )

    def test_save_not_author(self):
        self.browser.addHeader(
            'Authorization',
            'Basic %s:%s' % (TEST_USER_ID, TEST_USER_PASSWORD,)
        )
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/testapp/blocks/book')
        self.assertRaises(
            Unauthorized,
            self.browser.getControl('Save').click,
        )

    def test_save(self):
        self.browser.addHeader(
            'Authorization',
            'Basic %s:%s' % (SITE_OWNER_NAME, SITE_OWNER_PASSWORD,)
        )
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/testapp/blocks/book')
        self.browser.getControl(name='author').value = u"Victor Hugo"
        self.browser.getControl('Save').click()
        self.assertTrue("Victor Hugo" in self.browser.contents)

    def test_plone_security_anonymous(self):
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/testapp/blocks/action')
        self.assertRaises(
            ExecutionError,
            self.browser.getControl('Create a content').click
        )
        self.assertTrue('my-content' not in self.portal.objectIds())

    def test_plone_security_member(self):
        self.browser.addHeader(
            'Authorization',
            'Basic %s:%s' % (TEST_USER_ID, TEST_USER_PASSWORD,)
        )
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/testapp/blocks/action')
        self.assertRaises(
            ExecutionError,
            self.browser.getControl('Create a content').click
        )
        self.assertTrue('my-content' not in self.portal.objectIds())

    def test_plone_security_manager(self):
        self.browser.addHeader(
            'Authorization',
            'Basic %s:%s' % (SITE_OWNER_NAME, SITE_OWNER_PASSWORD,)
        )
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/testapp/blocks/action')
        if "Confirm action" in self.browser.contents:
            self.browser.getControl('Confirm action').click()
        self.browser.getControl('Create a content').click()
        self.assertTrue('my-content' in self.portal.objectIds())

    def test_pt_template(self):
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/testapp/blocks/knowledge')
        self.assertTrue('<p>Knowledge is power</p>\n<ul><li>France is bacon'
            '</li>\n<li>Francis Bacon</li></ul>\n<a href="http://localhost:'
            '55001/plone/@@rapido/testapp/blocks/knowledge">Home</a>'
            in self.browser.contents)

    def test_bad_pt_template(self):
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/testapp/blocks/bad')
        self.assertTrue('Rendering error\n - Expression: "boom/jokes"\n'
            ' - Location:   (line 2: col 25)' in self.browser.contents)

    def test_missing_template(self):
        try:
            self.browser.open(
                self.portal.absolute_url() + '/@@rapido/testapp/blocks/oops')
        except Exception, e:
            self.assertTrue(type(e) is zExceptions.NotFound)
        else:
            self.fail("Missing template should produce a NotFound exception.")

    def test_call(self):
        self.assertEquals(
            self.portal.restrictedTraverse('@@rapido-call')(
                'testapp/call/call_me', x=7, y=6),
            42
        )

    def test_access_other_app(self):
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/testapp/blocks/otherapp')
        self.assertTrue('Records=1' in self.browser.contents)
        self.browser.open(
            self.portal.absolute_url() + '/@@rapido/testapp/blocks/otherapp')
        self.assertTrue('Records=2' in self.browser.contents)



# -*- coding: utf-8 -*-
"""Installer for the rapido.plone package."""

from setuptools import find_packages
from setuptools import setup


long_description = (
    open('README.rst').read()
    + '\n' +
    'Contributors\n'
    '============\n'
    + '\n' +
    open('CONTRIBUTORS.rst').read()
    + '\n' +
    open('CHANGES.rst').read()
    + '\n')


setup(
    name='rapido.plone',
    version='1.0.3',
    description="Rapido for Plone",
    long_description=long_description,
    # Get more from http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 5.0",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
    ],
    keywords='Python Plone rapido',
    author='Eric BREHAULT',
    author_email='ebrehault@gmail.com',
    url='https://github.com/collective/rapido.plone',
    license='GPL',
    packages=find_packages('src', exclude=['ez_setup']),
    namespace_packages=['rapido'],
    package_dir={'': 'src'},
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'plone.api',
        'setuptools',
        'rapido.core',
        'rapido.souper',
        'souper.plone',
    ],
    extras_require={
        'test': [
            'plone.app.testing',
            'plone.app.contenttypes',
            'plone.app.robotframework[debug]',
        ],
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



import Globals
from plone.app.contentrules.rule import Rule
from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles
from plone.app.theming.interfaces import IThemeSettings
from plone.app.theming.utils import applyTheme
from plone.app.theming.utils import getTheme
from plone.contentrules.engine.interfaces import IRuleStorage
from plone.contentrules.rule.interfaces import IRuleAction
from plone.contentrules.rule.interfaces import IExecutable
from plone.registry.interfaces import IRegistry
from Products.statusmessages import STATUSMESSAGEKEY
from Products.statusmessages.adapter import _decodeCookieValue
import unittest2 as unittest
from zope.component import getUtility, getMultiAdapter
from zope.interface import implements, Interface

from rapido.plone.contentrule.action import Action, EditFormView
from rapido.plone.testing import RAPIDO_PLONE_FUNCTIONAL_TESTING


class DummyEvent(object):
    implements(Interface)

    def __init__(self, obj):
        self.object = obj


class TestRapidoAction(unittest.TestCase):

    layer = RAPIDO_PLONE_FUNCTIONAL_TESTING

    def setUp(self):
        # Enable debug mode always to ensure cache is disabled by default
        Globals.DevelopmentMode = True

        self.settings = getUtility(IRegistry).forInterface(IThemeSettings)
        self.settings.enabled = True
        theme = getTheme('rapido.plone.tests')
        applyTheme(theme)

        import transaction
        transaction.commit()
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def testRegistered(self):
        element = getUtility(IRuleAction, name='rapido.plone.Action')
        self.assertEqual('rapido.plone.Action', element.addview)
        self.assertEqual('edit', element.editview)

    def testInvokeAddView(self):
        element = getUtility(IRuleAction, name='rapido.plone.Action')
        storage = getUtility(IRuleStorage)
        storage[u'foo'] = Rule()
        rule = self.portal.restrictedTraverse('++rule++foo')

        adding = getMultiAdapter((rule, self.request), name='+action')
        addview = getMultiAdapter((adding, self.request), name=element.addview)

        addview.form_instance.update()
        content = addview.form_instance.create(data={
            'app': 'testapp',
            'block': 'rule',
            'method': 'hello',
        })
        addview.form_instance.add(content)

        e = rule.actions[0]
        self.assertTrue(isinstance(e, Action))
        self.assertEqual('testapp', e.app)
        self.assertEqual(e.summary,
            u'Call Rapido method hello from testapp/rule')

    def testInvokeEditView(self):
        element = getUtility(IRuleAction, name='rapido.plone.Action')
        e = Action()
        editview = getMultiAdapter((e, self.request), name=element.editview)
        self.assertTrue(isinstance(editview, EditFormView))

    def testExecute(self):
        e = Action()
        e.app = 'testapp'
        e.block = 'rule'
        e.method = 'hello'

        ex = getMultiAdapter(
            (self.portal, e, DummyEvent(self.portal)), IExecutable)
        self.assertEqual(True, ex())

    def testNotFound(self):
        e = Action()
        e.app = 'badapp'
        e.block = 'rule'
        e.method = 'hello'

        ex = getMultiAdapter(
            (self.portal, e, DummyEvent(self.portal)), IExecutable)
        ex()
        new_cookies = self.request.RESPONSE.cookies[STATUSMESSAGEKEY]
        messages = _decodeCookieValue(new_cookies['value'])
        self.assertEquals(messages[0].message,
            u'Rapido application badapp cannot be found.')
